/*
 * Ismail Yenigul's famous config reader/parser
 * This code derived from EnderUNIX isoqlog project.
 *
 * http://www.enderunix.org/isoqlog
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>

#include "loadconfig.h"

void loadconfig(char *cfgfile)	/* load isoqlog configuration file */
{
	FILE *fd = NULL;
	char buf[BIGBUFSIZE];
	char keyword[TINYBUFSIZE];
	char value[BIGBUFSIZE];
	int lenbuf = 0;
	char *cp1 = NULL, *cp2 = NULL;
	char *variables[] = { "Invalid",
		"WORKDIRPREFIX",
		"TEMPDIRPREFIX",
		"LOGFILE",
		"debug_level",
		"paronia_level",
		"drop_empty_to",
		"forbidden_header_lines",
		"enable_blackhole",
		"enable_quarantine",
		"enable_save_all",
		"enable_wblist",
		"enable_subject_filter",
		"enable_body_filter",
		"enable_attach_filter",
		"enable_virus_prog",
		"enable_filter_prog",
		"MIME_PROG",
		"MIME_PROG_OK_RET",
		"MIME_PROG_ERR_RET",
		"VIRUS_PROG",
		"VIRUS_PROG_OK_RET",
		"VIRUS_PROG_VIRUS_RET",
		"VIRUS_PROG_ERR_RET",
		"FILTER_PROG",
		"FILTER_PROG_OK_RET",
		"FILTER_PROG_VIRUS_RET",
		"FILTER_PROG_ERR_RET",
		"QUEUE_PROG"
	};

	int i, j, key, line, keyword_nums = sizeof(variables)/sizeof(char *);

	if ((fd = fopen(cfgfile, "r")) == NULL) {
		fprintf(stderr, "loadconfig: cannot open %s configuration file %s, exiting...\n", PROGNAME, cfgfile);
		exit(-1);
	}
	line = 0;
	while ((fgets(buf, BIGBUFSIZE, fd)) != NULL) {
		line++;
		if (buf[0] == '#')
			continue;
		if ((lenbuf = strlen(buf)) <= 1)
			continue;
		cp1 = buf;
		cp2 = keyword;
		j = 0;
		while (isspace((int)*cp1) && ((cp1 - buf) < lenbuf)) 
			cp1++;
		while(isgraph((int)*cp1) && *cp1 != '=' && (j++ < TINYBUFSIZE - 1) && (cp1 - buf) < lenbuf)
			*cp2++ = *cp1++;
		*cp2 = '\0';
		cp2 = value;
		while ((*cp1 != '\0') && (*cp1 !='\n') && (*cp1 !='=') && ((cp1 - buf) < lenbuf))
			cp1++;
		cp1++; 
		while (isspace((int)*cp1) && ((cp1 - buf) < lenbuf))
			cp1++; 
		if (*cp1 == '"') 
			cp1++;
		j = 0;
		while ((*cp1 != '\0') && (*cp1 !='\n') && (*cp1 !='"') && (j++ < TINYBUFSIZE - 1) && ((cp1 - buf) < lenbuf))
			*cp2++ = *cp1++;
		*cp2-- = '\0';
		if (keyword[0] =='\0' || value[0] =='\0')
			continue;
		key = 0;
		for (i = 0; i < keyword_nums; i++) {
			if ((strncmp(keyword, variables[i], TINYBUFSIZE)) == 0) {
				key = i;
				break;
			}
		}

		switch(key) {
		case 0:
			fprintf(stderr, "illegal keyword in config file: %s\n", keyword);
			break;
		case 1:
			strncpy(WORKDIRPREFIX, value, TINYBUFSIZE-1);
			break;
		case 2:
			strncpy(TEMPDIRPREFIX, value, TINYBUFSIZE-1);
			break;
		case 3:
			strncpy(LOGFILE, value, BIGBUFSIZE-1);
			break;
		case 4:
			debug_level = atoi(value);
			break;
	 	case 5:
	 		paronia_level = atoi(value);
	 		break;
		case 6:
			drop_empty_to = atoi(value);
			break;
		case 7:
			strncpy(forbidden_header_lines, value, BIGBUFSIZE-1);
			break;
		case 8:
			enable_blackhole = atoi(value);
			break;
	 	case 9:
	 		enable_quarantine = atoi(value);
	 		break;
		case 10:
			enable_save_all = atoi(value);
			break;
	 	case 11:
		 	enable_wblist = atoi(value);
		 	break;
		case 12:
			enable_subject_filter = atoi(value);
		 	break;
		case 13:
			enable_body_filter = atoi(value);
			break;
		case 14:
			enable_attach_filter = atoi(value);
			break;
	 	case 15:
	 		enable_virus_prog = atoi(value);
	 		break;
		case 16:
			enable_filter_prog = atoi(value);
			break;
	 	case 17:
	 		strncpy(MIME_PROG, value, BIGBUFSIZE-1);
	 		break;
	 	case 18:
	 		MIME_PROG_OK_RET = atoi(value);
	 		break;
	 	case 19:
	 		MIME_PROG_ERR_RET = atoi(value);
	 		break;
		case 20:
			strncpy(VIRUS_PROG, value, BIGBUFSIZE-1);
		 	break;
		case 21:
			VIRUS_PROG_OK_RET = atoi(value);
		 	break;
		case 22:
			VIRUS_PROG_VIRUS_RET = atoi(value);
		 	break;
		case 23:
			VIRUS_PROG_ERR_RET = atoi(value);
		 	break;
		case 24:
			strncpy(FILTER_PROG, value, BIGBUFSIZE-1);
		 	break;
		case 25:
			FILTER_PROG_OK_RET = atoi(value);
		 	break;
		case 26:
			FILTER_PROG_VIRUS_RET = atoi(value);
		 	break;
		case 27:
			FILTER_PROG_ERR_RET = atoi(value);
		 	break;
		case 28:
			strncpy(QUEUE_PROG, value, BIGBUFSIZE-1);
		 	break;
		}
	}
	fclose(fd);
}

