/*
 * Copyright (C) 2004 EnderUNIX Software Development Team @ Turkey
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: main.c,v 1.45 2005/08/14 20:15:38 simsek Exp $
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <dirent.h>

#include "defs.h"
#include "main.h"
#include "loadconfig.h"
#include "log.h"
#include "smtp.h"
#include "wblist.h"
#include "toolkit.h"
#include "scanengine.h"
#include "config.h"

extern int errno;
int fd;

int
main(int argc, char *argv[], char *env[])
{
	int	i, ret;
	char	*exec_cmdline[10];
	char	buf[TINYBUFSIZE] = "";
	DIR	*dirp;
	struct dirent *direntp;
	FILE*	fd2;

	while((ret = getopt(argc, argv, "v")) != -1) {
		switch(ret) {
		case 'v':
			fprintf(stdout, "qSheff %s - qmail spam/antivirus wrapper\n", VERSION);
			fprintf(stdout, "http://www.enderunix.org/qsheff/\n\n");
			fprintf(stdout, "EnderUNIX Software Development Team @ Turkey\n");
			fprintf(stdout, "		   http://www.enderunix.org/\n\n");
			exit(0);
			break;
		}
	}

	memset(rfc821_recvfrom, 0, sizeof(rfc821_recvfrom));
	if(getenv("TCPREMOTEIP") != NULL) strncpy(rfc821_recvfrom, getenv("TCPREMOTEIP"), sizeof(rfc821_recvfrom)-1);

	if(strlen(rfc821_recvfrom) < 7) strncpy(rfc821_recvfrom, "localuser", sizeof(rfc821_recvfrom)-1);

	loadconfig(CFGFILE);
	umask(0022);
	gen_dir_suffix(dir_suffix);
	snprintf(buf, TINYBUFSIZE-1, "X-Mail-Scanner: Scanned by %s %s (http://www.enderunix.org/qsheff/)\n\r", PROGNAME, VERSION);

	snprintf(workdir, BIGBUFSIZE-1, "%s/%s", WORKDIRPREFIX, dir_suffix);
	if(mkdir(workdir, 0755) == -1) {
#ifdef DEBUG
		perror("mkdir workdir");
#endif
		_exit(EX_TEMPORARY);
	}
	if(chdir(workdir) != 0) {
#ifdef DEBUG
		perror("chdir workdir");
#endif
		rm_dir(workdir);
		_exit(EX_TEMPORARY);
	}
	if((fd = open("mesg", O_WRONLY|O_CREAT|O_TRUNC,0644)) == - 1) {
#ifdef DEBUG
		perror("open mesg");
#endif
		rm_dir(workdir);
		_exit(EX_TEMPORARY);
	}

	write(fd, buf, strlen(buf)-1);	

#ifdef DEBUG
	printf("\n+++\n");
	printf("+++ %s %s ready for input\n", PROGNAME, VERSION);
	printf("+++\n\n");
#endif

	while((ret = read(0, buffer, sizeof(buffer)-1)) > 0) {
		if(write(fd, buffer, ret) == -1) {
#ifdef DEBUG
			perror("write mesg");
#endif
			close(fd);
			rrm_dir(workdir);
			_exit(EX_READ);
		}
	}
	close(fd);

	snprintf(tempdir, BIGBUFSIZE-1, "%s/%s", TEMPDIRPREFIX, dir_suffix);
	if(mkdir(tempdir, 0755) == -1) {
#ifdef DEBUG
		perror("mkdir tempdir");
#endif
		close(fd);
		rrm_dir(workdir);
		_exit(EX_TEMPORARY);
	}

/************************ Parse Header ************************/
	parse_cmdline(MIME_PROG, tempdir, exec_cmdline);
	ret = run_ext_prog(exec_cmdline);
	for(i=0;(exec_cmdline[i] != NULL);i++) free(exec_cmdline[i]);
#ifdef DEBUG
	printf("Mime parser '%s' exited with %d\n", MIME_PROG, ret);
#endif
	if((ret != MIME_PROG_OK_RET) || (ret == MIME_PROG_ERR_RET)) {
		clean_exit("TP00102");
		_exit(EX_TEMPORARY);
	}
	if(chdir(tempdir) != 0) {
#ifdef DEBUG
		perror("chdir tempdir");
#endif
		clean_exit("TP00109");
		_exit(EX_TEMPORARY);
	}

	if((ret = parse_header() == -1)) {
		clean_exit("TP00117");
		_exit(EX_TEMPORARY);
	}

/************************ Check header ************************/
	if(strncmp(rfc821_from, "unknown", sizeof(rfc821_from)) == 0) {
#ifdef DEBUG
		printf("empty from\n");
#endif
		clean_exit("PH00137");
		_exit(0);
	}

	if(drop_empty_to == 1)
		if(strncmp(rfc821_to, "unknown", sizeof(rfc821_to)) == 0) {
#ifdef DEBUG
			printf("empty to\n");
#endif
			clean_exit("PH00138");
			_exit(0);
		}

	if(strlen(forbidden_header_lines) > 2) {
		if((fd2 = fopen("_headers_", "r")) == - 1) {
#ifdef DEBUG
			printf("open _header_\n");
#endif
			clean_exit("TJ00139");
			_exit(0);
		}
		while(fgets(buffer, sizeof(buffer)-1, fd2) > 0) {
			ret = 0;
			while(forbidden_header_lines[ret] != '\0') {
				i = 0;
				while((forbidden_header_lines[ret] != '\0') && (forbidden_header_lines[ret] != '|'))
					buf[i++] = forbidden_header_lines[ret++];
				buf[i] = '\0';
				if(strstr(buffer, buf)) {
#ifdef DEBUG
					printf("forbidden header\n");
#endif
					fclose(fd2);
					clean_exit("PJ00139");
					_exit(0);
				}
			}
		}
		fclose(fd2);
	}

	if((strncmp(rfc821_recvfrom, "localuser", sizeof(rfc821_recvfrom)) == 0) && (strncmp(rfc821_from, MAILERDAEMON, sizeof(rfc821_from)) == 0)) {
#ifdef DEBUG
		printf("bounced message from: %s, to: %s\n", rfc821_from, rfc821_to);
		printf("All of the filters are disabled.\n");
#endif
		enable_wblist = 0;
		enable_subject_filter = 0;
		enable_body_filter = 0;
		enable_virus_prog = 0;
		enable_attach_filter = 0;
	}

/************************ White/Black List ************************/
	if(enable_wblist == 1) {
		ret = read_wblist();
#ifdef DEBUG
		printf("wblist exited with %d\n", ret);
#endif
		if(ret == 0) {
			if(strncmp(sfe_ruleline, "ACCEPT", sizeof(sfe_ruleline)) == 0) {
				enable_subject_filter = 0;
				enable_body_filter = 0;
				enable_attach_filter = 0;
			}
			else if(strncmp(sfe_ruleline, "REJECT", sizeof(sfe_ruleline)) == 0) {
				clean_exit("PW00133");
				do_exit();
			}
			else if(strncmp(sfe_ruleline, "DISCARD", sizeof(sfe_ruleline)) == 0) {
				clean_exit("PW00137");
				_exit(0);
			}
		}
		else if(ret == -1) {
			clean_exit("TW00142");
			_exit(EX_TEMPORARY);
		}
	}

	if(enable_attach_filter == 1) {
		if(load_attachlist() == -1) {
	#ifdef DEBUG
			printf("Couldnt load attach list\n");
	#endif
			clean_exit("TA00178");
			_exit(EX_TEMPORARY);
		}
	
		ret = attach_filter();
#ifdef DEBUG
		printf("Attach filter ret: %d\n", ret);
#endif
		if(ret > 0) { /* if found denied file attached */
			clean_exit("PA00174");
			do_exit();
		}
		else if(ret == -1) {
			clean_exit("TA00178");
			_exit(EX_TEMPORARY);
		}
	}

	if((enable_subject_filter == 1) || (enable_body_filter == 1))
		if((rule_sp = load_rulelist()) == NULL) {
#ifdef DEBUG
			printf("Couldnt load rule list\n");
#endif
			clean_exit("TS00178");
			_exit(EX_TEMPORARY);
		}

/************************ Subject Filter ************************/
	if(enable_subject_filter == 1) {
		ret = subj_filter(rfc821_subj);
#ifdef DEBUG
		printf("subj_filter exited with %d\n", ret);
#endif
		if(ret > 0) {
			clean_exit("PS00174");
			do_exit();
		} /* Found spam */
		else if(ret == -1) {
			clean_exit("TS00178");
			_exit(EX_TEMPORARY);
		}
	}

/************************ Body Filter ************************/
	if(enable_body_filter == 1) {
		if((dirp = opendir(tempdir)) == NULL) {
			clean_exit("TB00178");
			_exit(EX_TEMPORARY);
		}
		ret = 0;
		while((direntp = readdir(dirp)) != NULL) {
			if(((strlen(direntp->d_name) > 8)  && (strncmp(direntp->d_name, "textfile", 8) == 0)) || (strncmp(direntp->d_name, rfc821_name, strlen(direntp->d_name)) == 0)) {
				ret = body_filter(direntp->d_name);
#ifdef DEBUG
				printf("body_filter for %s exited with %d\n", direntp->d_name, ret);
#endif
		
				if(ret > 0) { /* if found spam */
					clean_exit("PB00174");
					do_exit();
				}
			}
		}
		if(ret == -1) {
			clean_exit("TB00178");
			 _exit(EX_TEMPORARY);
		}
	}

/************************ Virus Program ************************/
	if(enable_virus_prog == 1) {
		parse_cmdline(VIRUS_PROG, tempdir, exec_cmdline);
		ret = run_ext_prog(exec_cmdline);
		for(i=0;(exec_cmdline[i] != NULL);i++) free(exec_cmdline[i]);
#ifdef DEBUG
		printf("Antivirus '%s' exited with %d\n", VIRUS_PROG, ret);
#endif
		if(ret == VIRUS_PROG_VIRUS_RET) {
			clean_exit("PV00137");
			do_exit();
		}
		else if(ret == VIRUS_PROG_ERR_RET) {
			clean_exit("TV00142");
			_exit(EX_TEMPORARY);
		}
	}

/************************ Filter Program ************************/
	if(enable_filter_prog == 1) {
		parse_cmdline(FILTER_PROG, tempdir, exec_cmdline);
		ret = run_ext_prog(exec_cmdline);
		for(i=0;(exec_cmdline[i] != NULL);i++) free(exec_cmdline[i]);
#ifdef DEBUG
		printf("Filter program '%s' exited with %d\n", FILTER_PROG, ret);
#endif
		if(ret == FILTER_PROG_VIRUS_RET) {
			clean_exit("PF00137");
			do_exit();
		}
		else if(ret == FILTER_PROG_ERR_RET) {
			clean_exit("TF00142");
			_exit(EX_TEMPORARY);
		}
	}

/************************ qmail-queue ************************/
	if(chdir(workdir) != 0) {
#ifdef DEBUG
		perror("chdir workdir");
#endif
		clean_exit("TQ00151");
		_exit(EX_TEMPORARY);
	}
	if((fd = open("mesg", O_RDONLY)) == -1) {
#ifdef DEBUG
		perror("open mesg");
#endif
		clean_exit("TQ00158");
		_exit(EX_WRITE);
	}
	close(STDIN);
	dup2(fd, STDIN);
#ifdef DEBUG
	printf("\nqSheff is going to wake qmail-queue...\n");
#endif
	
	parse_cmdline(QUEUE_PROG, NULL, exec_cmdline);
	ret = run_ext_prog(exec_cmdline);
	for(i=0;(exec_cmdline[i] != NULL);i++) free(exec_cmdline[i]);

#ifdef DEBUG
	printf("qmail-queue '%s' exited with %d...\n\n", exec_cmdline[0], ret);
#endif
	close(fd);
	clean_exit("OO00200");
	_exit(ret);
}

/*------------------------------- Functions -------------------------------*/

static int
do_exit() {
	if(enable_blackhole == 0)
		_exit(EX_PERMANENT);
	else
		_exit(0);
}

static int
clean_exit(const char *errcode) {
	logging(errcode);
	close(fd);
	rrm_dir(tempdir);
	if((enable_quarantine == 0) && (enable_save_all == 0)) rrm_dir(workdir);
	
	return 0;
}

static char *
parse_cmdline(char *line, char *path, char *exec_cmdline[10]) {
	int	ret	;
	char	*part	;

	ret = 0;
	part = strtok(line, " ");
	if((exec_cmdline[0] = malloc(strlen(part)+1)) == NULL) {
		clean_exit("TC00269");
		_exit(EX_TEMPORARY);
	}
	strcpy(exec_cmdline[ret++], part);
	while((part = strtok(NULL, " ")) != NULL) {
		if((exec_cmdline[ret] = malloc(strlen(part)+1)) == NULL) {
			clean_exit("TC00275");
			exit(EX_TEMPORARY);
		}
		memset(exec_cmdline[ret], 0, strlen(part)+1);
		strcpy(exec_cmdline[ret++], part);
		if(ret >= 10) break;
	}
	if(path != NULL) {
		if((exec_cmdline[ret] = malloc(strlen(path)+1)) == NULL) {
			clean_exit("TC00284");
			_exit(EX_TEMPORARY);
		}
		memset(exec_cmdline[ret], 0, strlen(path)+1);
		strcpy(exec_cmdline[ret++], path);
	}
	exec_cmdline[ret] = NULL;

	return exec_cmdline[0];
}

static int
run_ext_prog(char *cmdline[]) {
	pid_t	pid				;
	int		child_status	;

	if((pid = fork()) == -1) {
#ifdef DEBUG
		printf("fork error: %s\n", strerror(errno));
#endif
		return -700;
	}

	if(pid == 0) {
		if(execve(cmdline[0], cmdline, NULL) == -1) {
#ifdef DEBUG
			printf("execve error: %s\n", strerror(errno));
#endif
			return -701;
		}
	}
	else {
		if((wait(&child_status)) == -1) return -703;

		/* Check if the child died with a signal */
		if(WIFSIGNALED(child_status)) return -705;
		
		/* Check if the child stopped with a signal */
		if(WIFSTOPPED(child_status)) return -707;

		/* Successfull return, take return code */
		if(WIFEXITED(child_status)) return WEXITSTATUS(child_status);
	}

	return -709; /* Normally, program never come here */
}

static rulelist		*
load_rulelist() {
	int		i				;
	FILE		*fd				;
	rulelist	*ll, *sp			;
	char		ruleline[TINYBUFSIZE]		;

	memset(ruleline, 0, TINYBUFSIZE);

	if((fd = fopen(RULEFILE, "r")) == NULL) {
#ifdef DEBUG
		printf("Couldn't load rule file %s: %s\n", RULEFILE, strerror(errno));
#endif
		return NULL;
	}

	if((ll = (rulelist *) malloc(sizeof(rulelist))) == NULL) {
#ifdef DEBUG
		printf("Couldn't allocate memory for rule list: %s\n", strerror(errno));
#endif
		return NULL;
	}

	sp = ll;
	while((fgets(ruleline, TINYBUFSIZE-1, fd) != 0) && (!feof(fd))) {
		if(((ruleline[0] == 'b') || (ruleline[0] == 's') || (ruleline[0] == 'a')) && (ruleline[1] == ':') && (strlen(ruleline) > 4)) {
			ll->attr = ruleline[0];

			for(i=0;i<strlen(ruleline)-2;i++) ll->ruleline[i] = ruleline[i+2];
			ll->ruleline[i-1] = '\0';
			
			ll->next = (rulelist *) malloc(sizeof(rulelist));
			ll = ll->next;
		}
		memset(ruleline, 0, TINYBUFSIZE);
	}
	
	ll = NULL;

	fclose(fd);

	return sp;
}

static	int
load_attachlist() {
	int	i = 0				;
	FILE	*fd				;
	char	pattern[128]	;

	if((fd = fopen(ATTACHFILE, "r")) == NULL) {
#ifdef DEBUG
		printf("Couldn't load attach list file %s: %s\n", ATTACHFILE, strerror(errno));
#endif
		return -1;
	}

	while((i < 128) && (!feof(fd))) {
		memset(pattern, 0, sizeof(pattern));
		if((fscanf(fd, "%127s", pattern) < 0 ) && (!feof(fd))) {
#ifdef DEBUG
			printf("Couldn't read attach list file %s: %s\n", ATTACHFILE, strerror(errno));
#endif
			return -1;
		}
		if(strlen(pattern) > 1) {
			strncpy(attach_list[i], pattern, sizeof(attach_list[i])-1);
			i++;
		}
	}

	fclose(fd);

	return 0;
}




