/*
 * Copyright (C) 2005 Baris Simsek, <simsek@enderunix.org>
 *                        http://www.enderunix.org/simsek/
 *
 * $Id: scanengine.c,v 1.21 2005/08/14 20:18:07 simsek Exp $
 *
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include "defs.h"
#include "loadconfig.h"
#include "scanengine.h"
#include "config.h"

extern int errno;

int
subj_filter(char *subject) {
	int		i		;
	char		*ruleline	;
	rulelist	*tmp		;
	char		*ruleword	;

	strncpy(sfe_spamword, subject, sizeof(sfe_spamword)-1);

	tmp = rule_sp;
	while(tmp != NULL) {
		if((tmp->attr == 's') || (tmp->attr == 'a')) {
			ruleline = tmp->ruleline;
			strncpy(sfe_ruleline, ruleline, sizeof(sfe_ruleline)-1);
			while((ruleword = get_ruleword(ruleline)) != NULL) {
				i = 0;
				if(is_there(subject, ruleword) == 0) break;
				i = 1;
				if((ruleline = strip_ruleline(ruleline)) == NULL) break;
			}
			if(i == 1) return 1;
		}
		tmp = tmp->next;
	}

	return 0;
}

int
body_filter(char *filepath) {
	int		i		;
	FILE		*fd		;
	char		*ruleword	;
	rulelist	*tmp1		;
	wordlist	*tmp2		;
	wordlist	*bodyp		;
	char		*ruleline	;

	if((fd = fopen(filepath, "r")) == NULL) {
#ifdef DEBUG
		printf("[body_filter] Couldn't open body file %s: %s\n", filepath, strerror(errno));
#endif
		return -1;
	}

	bodyp = (wordlist *) malloc(sizeof(wordlist));
	tmp2 = bodyp;
	i = 0;
	while((feof(fd) == 0) && (i++ < MAXWORDCOUNT)) {
		memset(tmp2->word, 0, sizeof(tmp2->word));
		ruleword = tmp2->word;
		*ruleword++ = ' ';
		fscanf(fd, "%61s", ruleword);
		ruleword[strlen(ruleword)] = ' ';
		ruleword[strlen(ruleword)+1] = '\0';
		tmp2->next = (wordlist *) malloc(sizeof(wordlist));
		tmp2 = tmp2->next;
		memset(tmp2->word, 0, sizeof(tmp2->word));
	}

	fclose(fd);

	tmp1 = rule_sp;
	while(tmp1 != NULL) {
		if((tmp1->attr == 'b') || (tmp1->attr == 'a')) {
			ruleline = tmp1->ruleline;
			strncpy(sfe_ruleline, tmp1->ruleline, sizeof(sfe_ruleline)-1);
			while((ruleword = get_ruleword(ruleline)) != NULL) {
				i = 0;
				if(search_in_list(bodyp, ruleword) == 0) break;
				i = 1;
				if((ruleline = strip_ruleline(ruleline)) == NULL) break;
			}
			if(i == 1) return 1;
		}
		tmp1 = tmp1->next;
	}

	return 0;
}

int
search_in_list(wordlist *listp, char *ruleword) {
	while(listp->word[0] != '\0') {
		strncpy(sfe_spamword, listp->word, sizeof(sfe_spamword)-1);
		if(is_there(listp->word, ruleword) > 0) return 1;
		listp = listp->next;
	}

	return 0;
}

char	*
strip_ruleline(char *ruleline) {
	char	ch;

	while(*ruleline != '\0') {
		ch = *ruleline;
		ruleline++;
		if((ch == ')') && (*ruleline == '(')) break;
	}

	if(strlen(ruleline) < 1) return NULL;

	return ruleline;
}

char	*
strip_line(char *line) {
	/* Strip spaces */
	while(((*line == ' ') || (*line == '\t')) && (*line != '\0')) line++;
	/* Go to next word */
	while((*line != '\0') && (*line != ' ') && (*line != '\t')) line++;

	return line;
}

int
attach_filter() {
	int		i, j			;
	DIR 		*dirp			;
	struct dirent	*direntp		;
	struct stat	statbuf			;
	char		filename[TINYBUFSIZE]	;
	char		str[TINYBUFSIZE]	;

	memset(filename, 0, TINYBUFSIZE);

	if((dirp = opendir(".")) == NULL) return -1;

	while((direntp = readdir(dirp)) != NULL) {
		if(strlen(direntp->d_name) > TINYBUFSIZE-1) return -1;
		strncpy(filename, direntp->d_name, TINYBUFSIZE-1);

		if((strncmp(filename, "textfile", 8) !=0) && (strncmp(filename, "_headers_", 9) !=0) && (strcmp(filename, ".") !=0) && (strcmp(filename, "..") != 0)) {
			if(lstat(filename, &statbuf) == -1) {
				return -1;
			}
			if(!S_ISDIR(statbuf.st_mode)) {
				i = 0;
				while((i < TINYBUFSIZE) && (strlen(attach_list[i]) > 0)) {
					memset(str, 0, TINYBUFSIZE);
					if(strlen(attach_list[i]) <= strlen(direntp->d_name)) {
						/* Compare tails */
						for(j=0;j<strlen(attach_list[i]);j++)
							str[j] = filename[strlen(filename)-strlen(attach_list[i]+j)];
						str[j] = '\0';
						if(strstr(str, attach_list[i])) {
							strncpy(sfe_spamword, filename, sizeof(sfe_spamword)-1);
							strncpy(sfe_ruleline, str, sizeof(sfe_ruleline)-1);
							return 1;
						}
					}
					i++;
				}
			}
		}
		memset(filename, 0, sizeof(filename));
	}

	closedir(dirp);

	return 0;
}

int
is_there(char *input, char *ruleword) {
	int	ret;
	char	*tmp1, *tmp2	;

	while(*input != '\0') {
		tmp1 = input;
		tmp2 = ruleword;
		while((*tmp1 != *tmp2) && (*tmp2 != '?') && (*tmp2 != '\\')) {
			if(*tmp1 == '\0')
				return 0;
			else
				tmp1++;
		}

		while((*tmp1 != '\0') && (*tmp2 != '\0')) {
			ret = 0;
			if(*tmp2 == '?') {
				if(*(++tmp1) != *(++tmp2)) {
					tmp1--;
					break;
				}
			}
			else if(*tmp2 == '\\') {
				tmp2++;
				if(*tmp1 != *tmp2) break;
			}
			else if(*tmp1 != *tmp2) break;
			/* if equal */
			ret = 1;
			tmp1++;
			tmp2++;
		}

		if(((ret == 1) && (*tmp1 != '\0')) || ((ret == 1) && (*tmp1 == '\0') && (*tmp2 == '\0'))) return 1;
		if(*input != '\0') input++;
	}

	return 0;
}

char	*
get_ruleword(char *line) {
	int	i = 0	;
	char *ruleword	;

	ruleword = malloc(TINYBUFSIZE);
	memset(ruleword, 0, TINYBUFSIZE);

	line++;
	while((*line != ')') && (*line != '\0')) ruleword[i++] = *line++;
	ruleword[i] = '\0';

	return ruleword;
}

