#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <dirent.h>

#include "defs.h"
#include "toolkit.h"
#include "loadconfig.h"

extern int errno;

int
gen_dir_suffix(char *suffix)
{
  struct timeval time_str;

  gettimeofday(&time_str,(struct timezone *) 0);
  snprintf(suffix, TINYBUFSIZE - 2, "q-%ld-%ld-%ld", time_str.tv_sec, time_str.tv_usec, (long int)getpid());

  return 0;
}

char *get_date(void)
{
  char cur_date[20] = "";
  time_t now;
  struct tm *t;

  time(&now);
  t = localtime(&now);

  sprintf(cur_date, "%d/%02d/%02d %02d:%02d:%02d", (t->tm_year+1900), t->tm_mon+1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec);

  return cur_date;
}

int
rm_dir(const char *dirname)
{
  if(chdir(WORKDIRPREFIX) != 0) {
#ifdef DEBUG
    printf("chdir %s error: %s\n", WORKDIRPREFIX, strerror(errno));
#endif
  }

  if(rmdir(dirname) != 0) {
#ifdef DEBUG
    printf("rmdir %s error: %s\n", dirname, strerror(errno));
#endif
    return 2;
  }
  else
    return 0;
}


int
rrm_dir(const char *dirname)
{
  char newpath[BIGBUFSIZE];
  DIR *dirp;
  struct dirent *direntp;
  struct stat statbuf;

#ifdef DEBUG
    printf("Directory '%s' is removing...\n", dirname);
#endif

  /*
   * if chdir get error, Solaris will fail to remove current dir.
   * you should use a crontab script for cleaning
   */

  if(chdir(WORKDIRPREFIX) != 0) {
#ifdef DEBUG
    printf("chdir %s error: %s\n", WORKDIRPREFIX, strerror(errno));
#endif
  }

  if ((dirp = opendir(dirname)) == NULL) {
#ifdef DEBUG
    printf("Cannot open dir %s: %s\n", dirname, strerror(errno));
#endif
    return 2;
  }

  while((direntp = readdir(dirp)) != NULL) {
    if(strcmp(direntp->d_name, ".") !=0 && strcmp(direntp->d_name, "..") != 0) {
      snprintf(newpath, BIGBUFSIZE - 2, "%s/%s", dirname, direntp->d_name);
      if(lstat(newpath, &statbuf) == -1) {
        return 2;
      }
      if(S_ISDIR(statbuf.st_mode)) {
				rrm_dir(newpath);
      }
      else {
	if(unlink(newpath) == -1) {
#ifdef DEBUG
          printf("Cannot remove file %s: %s\n", newpath, strerror(errno));
#endif
	}
        else {
#ifdef DEBUG
          printf("  file '%s' has removed\n", newpath);
#endif
        }
      }
    }
  }

  closedir(dirp);

  if(rmdir(dirname) == -1) {
#ifdef DEBUG
    printf("cannot remove dir %s: %s\n", dirname, strerror(errno));
#endif
  }
  return 0;
}

int
removespaces(char *buf)
{
	char *cp = buf;
	char *sv = buf;
	int len;

	len = strlen(buf);

	for (; *buf != '\0' && *buf != '\r' && *buf != '\n' && ((buf - sv) < len); buf++)
		if (*buf == ' ')
			continue;
		else
			*cp++ = *buf;
	*cp = 0x0;
	return cp - sv;
}

